"""SNAPtoolbelt Mesh/Network Commands"""
from __future__ import print_function

import logging

import os
import sys

import progressbar

from arghandler import ArgumentHandler

from snap_profile.network import Network

import snapstraction.mesh
import snapstraction.node_finder

log = logging.getLogger(__name__)


def mesh(parser, context, args):
    """Mesh-related subcommands"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'ping': ping,
        'dm_ping': dm_ping,
        'find': find,
        'intercept': intercept_listen,
        'listen': listen,
        'sniff': sniff,
        'topology': topology,
    })

    log.info("mesh commands")
    log.info("mesh context: %r", context)
    log.info("mesh args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ping(parser, context, args):
    """Mcast ping the mesh"""
    handler = ArgumentHandler()

    handler.add_argument("-t", "--timeout", default=20, type=int,
                         help="Time to wait for responses")

    args = handler.parse_args(args)

    log.info("Ping the mesh")

    widgets = [progressbar.FormatLabel('Unicast Ping'), ' ',
               progressbar.FormatLabel('Found 0 ()'), ' ',
               progressbar.Percentage(), ' ',
               progressbar.Bar(), ' ',
               progressbar.Timer(), ' ',
               progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=args.timeout)
    progress.start()

    def on_progress(seconds, found_count, last_addr):
        widgets[2] = progressbar.FormatLabel('Found %d (%s)' % (found_count, last_addr))
        progress.update(seconds)

    def on_finished():
        progress.finish()

    with context.gus() as gus:
        result = gus.run_sync_with_data(args.timeout + 10, snapstraction.mesh.unicast_ping, args.timeout, context.output,
                                        progress_cb=on_progress, finished_cb=on_finished)
        context.output(result)


def dm_ping(parser, context, args):
    """Ping that uses dmcast for the response instead of unicast"""
    handler = ArgumentHandler()

    handler.add_argument("-t", "--timeout", default=20, type=int,
                         help="Time to wait for responses")
    args = handler.parse_args(args)

    log.info("Ping the mesh")

    widgets = [progressbar.FormatLabel('Directed Multicast Ping'), ' ',
               progressbar.FormatLabel('Found 0 ()'), ' ',
               progressbar.Percentage(), ' ',
               progressbar.Bar(), ' ',
               progressbar.Timer(), ' ',
               progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=args.timeout)
    progress.start()

    def on_progress(seconds, found_count, last_addr):
        widgets[2] = progressbar.FormatLabel('Found %d (%s)' % (found_count, last_addr))
        progress.update(seconds)

    def on_finished():
        progress.finish()

    with context.gus() as gus:
        result = gus.run_sync_with_data(args.timeout + 10, snapstraction.mesh.dmcast_ping, args.timeout, context.output,
                                        progress_cb=on_progress, finished_cb=on_finished)
        context.output(result)


def intercept_listen(parser, context, args):
    """Listen for intercepted node output"""
    handler = ArgumentHandler()
    handler.add_argument('-t', '--timeout', type=int, default=None, help='How many seconds to listen (default: forever)')
    args = handler.parse_args(args)

    log.info("Trying to listen for intercepted output")

    with context.gus() as gus:
        gus.run_sync(None, snapstraction.mesh.listen_for_intercepted_output, context.output, _timeout=args.timeout)


def find(parser, context, args):
    """find the target node"""
    handler = ArgumentHandler()
    handler.add_argument('target_address',
                         nargs='?',
                         default=None,
                         help="Node to find")
    handler.add_argument('-l',
                         '--list',
                         dest="list_file",
                         help="File containing list of nodes to find")
    handler.add_argument('-m', '--move', help="Profile to move nodes to.")
    args = handler.parse_args(args)

    if not args.target_address and not args.list_file:
        raise Exception(
            "Must provide either a target address or a file with a list of targets, one per line")

    # Need to allow a lot more time for finding nodes
    if context.timeout < 60:
        log.warn(
            "Bumping timeout to 300 seconds, finding nodes can take a while...")
        context.timeout = 300

    # do expansion of list file name/path
    list_file_path = os.path.join(
        os.path.curdir,
        os.path.expanduser(args.list_file)) if args.list_file else None

    # load the profile data to pass down
    move_to_network = Network.load(args.move) if args.move else None

    log.info("finding node(s), optionally moving to target network...")

    with context.gus() as gus:
        result = gus.run_sync_with_data(context.timeout, snapstraction.node_finder.finder,
                                        args.target_address, nodelist_path=list_file_path, move_to_network=move_to_network)
        context.output(result)


def topology(parser, context, args):
    """Take the topology of the mesh"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Get the topology of the mesh")

    with context.gus() as gus:
        result = gus.run_sync_with_data(context.timeout, snapstraction.mesh.poll_topology)
        context.output(result)


def sniff(parser, context, args):
    """Sniff traffic on the mesh"""
    handler = ArgumentHandler()
    handler.add_argument('channel', type=int, nargs='?', default=None, help='Override network channel to sniff')
    args = handler.parse_args(args)

    if args.channel:
        if args.channel < 0 or args.channel > 15:
            log.error('Bad channel number %d; only channels 0-15 are valid', args.channel)
            sys.exit(1)

    snapstraction.mesh.sniff(context.output, context.profile, context.network, args.channel)


def listen(parser, context, args):
    """Listen to packets received"""
    handler = ArgumentHandler()
    handler.add_argument('-t', '--timeout', type=int, default=None, help='How many seconds to listen (default: forever)')
    args = handler.parse_args(args)

    log.info("Listening to the mesh")

    with context.gus() as gus:
        gus.run_sync(None, snapstraction.mesh.listen, context.output, _timeout=args.timeout)
